# -*- coding: utf-8 -*-
"""
Created on Mon Feb 21 12:38:05 2022

@author: olavg
"""
#%% Import packages
import pandas as pd
import numpy as np
from scipy.interpolate import interp1d
import warnings
import matplotlib.pyplot as plt
from plotting import saveTex #Self made plot package
import os
#%% Startvariables that must be defined
path="Nqp31/"
name = "Suldal 1"
ys = [1.00,0.82,0.49,0.33,0.17,0.09,0.00] #wanted openings
markers = ["o","s","v","^","*","d","p","P"]

nbp,qbp,Tbp= 43.6,0.40,830.1
# nbp,qbp,Tbp = 57.46,0,0


plot_råpunkt=False
ms=5 #Markersize
n_punkt=4
#%% Initiation
#Initiate empty dataframe
df = pd.DataFrame(columns=['y','n','Q','T'])

#Initiate plot
plt.close('all')
fig,[axQ,axT]= plt.subplots(nrows=2,ncols=1,num="Plot of turbine curve for %s"%name,sharex=True)

#%% def
def split_and_interpolate(x,y):
    i = np.where(np.diff(np.diff(x)>0))[0]+1
    rep = np.ones_like(x,dtype=int)
    for e,ind in enumerate(i):
        rep[ind]=2
        i[e]+=e+1
    x = np.split(np.repeat(x,rep),i)
    y = np.split(np.repeat(y,rep),i)

    fq = []
    for i,(xi,yi) in enumerate(zip(x,y)):
        fq += [interp1d(xi, yi, kind= 'slinear',fill_value="extrapolate")]
    
    return x,y,fq
def linmove_end_to(x,x_end):
    x = np.asarray(x)
    x = x + (x-x[0])/(x[-1]-x[0]) * (x_end-x[-1])
    return x
def linmove_start_to(x,x_start):
    x = np.asarray(x)
    x = x + (x-x[-1])/(x[0]-x[-1]) * (x_start-x[0])
    return x
#%% Loop all openings y
ys = sorted(ys)
for mi,y in enumerate(ys):
    #Read csv to dataframe and add opening y
    dfQ = pd.read_csv("%sQy%03.0f.csv"%(path,y*100),decimal=",",sep=";",names=["n","Q"])
    dfT = pd.read_csv("%sTy%03.0f.csv"%(path,y*100),decimal=",",sep=";",names=["n","T"])
    
    xq = np.asarray(dfQ.n)
    yq = np.asarray(dfQ.Q)
    xt = np.asarray(dfT.n)
    yt = np.asarray(dfT['T'])
    
    x = np.sort(np.concatenate((xq,xt)))
    # xmin,xmax = np.amin(x),np.amax(x) #with extrapolation of ends
    xmin,xmax = np.amax([xq[0],xt[0]]),np.amin([xq[-1],xt[-1]]) #no extrapolation

    #indicies of local min max
    iq = np.where(np.diff(np.diff(xq)>0))[0]+1 
    it = np.where(np.diff(np.diff(xt)>0))[0]+1

    print("Q%.2f:"%y, iq)
    print("T:", it)
    if len(iq)>0 and len(it)>0 and len(it)==len(iq):
        
        xq,yq,fq = split_and_interpolate(xq,yq)
        xt,yt,ft = split_and_interpolate(xt,yt)

        n = np.array([])
        t,q =[],[]
        turnx=[]
        for ind in range(len(xq)):
            turnx += [np.mean([xq[ind][-1],xt[ind][-1]])]
            if ind==0:
                xi = linmove_start_to(xq[ind],xmin)
                xi = linmove_end_to(xi,turnx[-1])
                xi = xi[:-1]


            elif ind==1:                
                xi = linmove_start_to(xi,turnx[-2])
                xi = linmove_end_to(xq[ind],turnx[-1])
                xi = xi[:-1]

                
            elif ind==2:
                xi = linmove_start_to(xq[ind],turnx[-2])
                xi = linmove_end_to(xi,xmax)

            n = np.append(n,xi)
            q = np.append(q,fq[ind](xi))
            t = np.append(t,ft[ind](xi))        
    
    elif len(iq)==len(it)==0:
        fq = interp1d(xq, yq, kind= 'linear',fill_value="extrapolate")
        ft = interp1d(xt, yt, kind= 'linear',fill_value="extrapolate")
        n = x
        q = fq(n)
        t = ft(n)
        
    else:
        warnings.warn("Curves do not have the same number of turning point")
    #Plot
    axQ.plot(n,q,'-',marker=markers[mi],markevery=len(n)//n_punkt,label="Tilpasset Q, y=%.2f"%y)
    axT.plot(n,t,'-',marker=markers[mi],markevery=len(n)//n_punkt,label="Tilpasset T, y=%.2f"%y)
    if plot_råpunkt:
        axQ.plot(dfQ.n,dfQ['Q'], '-D',label="Råpunkt Q, y=%.2f"%y,markersize=ms)
        axT.plot(dfT.n,dfT['T'],'--o',label="Råpunkt T, y=%.2f"%y,markersize=ms)
    #Concatenate dataframe
    dfy = pd.DataFrame({'y':np.ones_like(n)*y,'n':n,'Q':q,'T':t})
    df = pd.concat([df,dfy],ignore_index=True)
#%% Fix legends   
axQ.set_ylabel(r'$\frac{Q}{Q_{11}}$',rotation=0)
axQ.legend(loc="upper left",bbox_to_anchor=(1.0, 1.05))
axT.set_ylabel("$\\frac{T}{T_{11}}$",rotation=0)
axT.legend(loc="upper left",bbox_to_anchor=(1.0, 1.05))
axT.set_xlabel("$\\frac{n}{n_{11}}$",rotation=0)
axT.grid()
axQ.grid()
plt.tight_layout()

#%% Scale data and remove NAN 
df['n'] = df['n']*nbp
df['Q'] = df['Q']*qbp
df['T'] = df['T']*Tbp
# df = df.where(pd.notnull(df), "")

#%% Print dataframe to file
with open("%sturbcurv_%s.txt"%(path,name), "w") as file1:
    l = 50
    #file1.write(";%s\n"%("*"*l))
    #l2 = (l-len(name))//2
    #file1.write(";%s%s%s\n"%(" "*l2,name," "*l2))
    #file1.write(";%s\n"%("*"*l))
    #file1.write(";\n")
    file1.write("; This is the turbine curve for %s kraftverk\n"%name)
    file1.write("; Given nbp=%.2f, Qbp=%.2f, Tbp=%.2f\n"%(nbp,qbp,Tbp))
    file1.write("; With turbine openings y = %s\n"%str(ys))
    file1.write("; Created by Marianne Aske for use in master's thesis spring 2022\n")
    #file1.write(";\n")
    #file1.write("<List>\n")
    header = df.to_string(index=False,max_rows=1,header=True)[1:].split("\n", 1)[0]
    file1.write(";" + header +"\n")
    file1.write("<List>\n")
    file1.write(df.to_string(index=False,header=False) +"\n")
    file1.write("</List>\n")
#%% Create plot
save_name="%s"%fig.get_label()
save_name="Suldal1"
saveTex(save_name, width=9, strict=False,scale_only_axis=False,vsep=1)
#Compile    
os.system("pdflatex --enable-write18 --extra-mem-top=1000000000 --synctex=1  %s.tex"%save_name)
#Open
os.startfile("%s.pdf"%save_name)